
#ifndef __AACPAM_H__
#define __AACPAM_H__


#include <stdio.h>
#include <string.h>
#include <math.h>
#include "keun_fft.h"


#define rpelev 2
#define rpelev2 16

/* size of each partition band, in barks: */
#define DELBARK .34


/* AAC values, results in more masking over MP3 values */
#define TMN 18		// MP3 value : 29
#define NMT 6		// MP3 value : 6

/* ISO values */
#define CONV1 (-.299)
#define CONV2 (-.43)

#define CBANDS		72
#define SBMAXL		51
#define SBMAXS		15

#define LN_TO_LOG10		(M_LN10/10)

#define PAMSIZE	2048
#define LONG_LEN	2048
#define SHORT_LEN	256

#define MIN(X,Y)	((X) > (Y) ? (Y) : (X))
#define MAX(X,Y)	((X) > (Y) ? (X) : (Y))

// Block Switching Window Type
#define LONG_WINDOW         0
#define LONG_START_WINDOW   1
#define SHORT_WINDOW        2
#define LONG_STOP_WINDOW    3

// FFT window Type
#define HANNING_WINDOW		0
#define HAMMING_WINDOW		1
#define BLACKMAN_WINDOW		2
#define RECT_WINDOW			3



class KeunPAM {
public:
	KeunPAM(int sf);
	~KeunPAM();

	int PsychoAcousticModel(float *PCMbuf,
						float PEntropy[2],
						float *MSR[2],
						long blocktype_d[2],
						int nCh);

	long *getCBwidthTab();
	long getNumSubBand() { return SBMAXL; }

	void setFFTWinType(int type);

protected:
	void CreatePAM();
	void DeletePAM();
	float S3Func(float bark);
	int parameterRead(float sfreq);
	int PsymodelInit();
	float Freq2Bark(float freq);
	float ATHformula(float f);

protected:
	float *EnergyL;
	float **EnergyS;
	float *AxSav[2][2];
	float *BxSav[2][2];
	float *RxSav[2][2];
	float *AxSavShort[2][2];
	float *BxSavShort[2][2];
	float *RxSavShort[2][2];
	float *CwLong;
	float **Cw;
	float PE[2];
	float PEs[2][8];
	float PEntropy[2];
	float *EnL[2];
	float *ThmL[2];
	float **EnS[2];
	float **ThmS[2];

	int CwUpper;
	int CwLower;
	int *NumLinesL;
	int *NumLinesS;
	int NumPartL;
	int NumPartS;
	int **S3Ind;
	int **S3Ind_S;
	float **S3_l;
	float **S3_s;
	float *Nb_1[2];
	float *Nb_S1[2];

	long BlkTypeOld[2];
	float *ATHcb;
	float *ATHcbS;
	float *SNR_s;
	float CwLimit;

	float *EnOldL[2];
	float *ThmOldL[2];
	float **EnOldS[2];
	float **ThmOldS[2];

	long *CBwidthL;
	long *CBwidthS;

	unsigned char SfIdx;

	long NumSfbL;
	long NumSfbS;
	long SampleRate;

	float *fftWinLong;
	float *fftWinShort;

	KeunFFT *longFFT;
	KeunFFT *shortFFT;

};


#endif
